aboutsummaryrefslogtreecommitdiff
path: root/gold/target-select.cc
diff options
context:
space:
mode:
authorIan Lance Taylor <ian@airs.com>2010-02-12 03:23:26 +0000
committerIan Lance Taylor <ian@airs.com>2010-02-12 03:23:26 +0000
commit114dfbe13efea1a1405dbcf5d9254e59d309382f (patch)
treec9a9058321b21712c1502a2bbc05225b370f7a23 /gold/target-select.cc
parent2310652a4f4b6568a96583e6930904e7c25efdcd (diff)
downloadgdb-114dfbe13efea1a1405dbcf5d9254e59d309382f.zip
gdb-114dfbe13efea1a1405dbcf5d9254e59d309382f.tar.gz
gdb-114dfbe13efea1a1405dbcf5d9254e59d309382f.tar.bz2
* gold-threads.h (class Once): Define.
(class Initialize_lock): Rewrite as child of Once. * gold-threads.cc (class Once_initialize): Define. (once_pointer_control): New static variable. (once_pointer, once_arg): New static variables. (c_run_once): New static function. (Once::Once, Once::run_once, Once::internal_run): New functions. (class Initialize_lock_once): Remove. (initialize_lock_control): Remove. (initialize_lock_pointer): Remove. (initialize_lock_once): Remove. (Initialize_lock::Initialize_lock): Move to gold-threads.h. (Initialize_lock::initialize): Rewrite. (Initialize_lock::do_run_once): New function. * archive.cc (Archive::interpret_header): Only clear name if it is not already empty. * fileread.cc: Include "gold-threads.h" (file_counts_lock): New static variable. (file_counts_initialize_lock): Likewise. (File_read::release): Only increment counts when using --stats. Use a lock around the increment. * parameters.cc (class Set_parameters_target_once): Define. (set_parameters_target_once): New static variable. (Parameters::Parameters): Move here from parameters.h. (Parameters::set_target): Rewrite. (Parameters::set_target_once): New function. (Parameters::clear_target): Move here and rewrite. * parameters.h (class Parameters): Update declarations. Add set_parameters_target_once_ field. (Parameters::Parameters): Move to parameters.cc. (Parameters::clear_target): Likewise. * readsyms.cc (Read_symbols::do_group): Create a Start_group task. (Start_group::~Start_group): New function. (Start_group::is_runnable): New function. (Start_group::locks, Start_group::run): New functions. (Finish_group::run): Change saw_undefined to size_t. * readsyms.h (class Start_group): Define. (class Finish_group): Change saw_undefined_ field to size_t. (Finish_group::Finish_group): Remove saw_undefined and this_blocker parameters. Change all callers. (Finish_group::set_saw_undefined): New function. (Finish_group::set_blocker): New function. * symtab.h (class Symbol_table): Change saw_undefined to return size_t. Change saw_undefined_ field to size_t. * target-select.cc (Set_target_once::do_run_once): New function. (Target_selector::Target_selector): Initialize set_target_once_ field. Don't initialize lock_ and initialize_lock_ fields. (Target_selector::instantiate_target): Rewrite. (Target_selector::set_target): New function. * target-select.h (class Set_target_once): Define. (class Target_selector): Update declarations. Make Set_target_once a friend. Remove lock_ and initialize_lock_ fields. Add set_target_once_ field.
Diffstat (limited to 'gold/target-select.cc')
-rw-r--r--gold/target-select.cc37
1 files changed, 22 insertions, 15 deletions
diff --git a/gold/target-select.cc b/gold/target-select.cc
index dcd3017..859bc3b 100644
--- a/gold/target-select.cc
+++ b/gold/target-select.cc
@@ -1,6 +1,6 @@
// target-select.cc -- select a target for an object file
-// Copyright 2006, 2007, 2008 Free Software Foundation, Inc.
+// Copyright 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
// Written by Ian Lance Taylor <iant@google.com>.
// This file is part of gold.
@@ -39,6 +39,14 @@ gold::Target_selector* target_selectors;
namespace gold
{
+// Class Set_target_once.
+
+void
+Set_target_once::do_run_once(void*)
+{
+ this->target_selector_->set_target();
+}
+
// Construct a Target_selector, which means adding it to the linked
// list. This runs at global constructor time, so we want it to be
// fast.
@@ -46,32 +54,31 @@ namespace gold
Target_selector::Target_selector(int machine, int size, bool is_big_endian,
const char* bfd_name)
: machine_(machine), size_(size), is_big_endian_(is_big_endian),
- bfd_name_(bfd_name), instantiated_target_(NULL), lock_(NULL),
- initialize_lock_(&this->lock_)
-
+ bfd_name_(bfd_name), instantiated_target_(NULL), set_target_once_(this)
{
this->next_ = target_selectors;
target_selectors = this;
}
// Instantiate the target and return it. Use a lock to avoid
// instantiating two instances of the same target.
Target*
Target_selector::instantiate_target()
{
- // We assume that the pointer will either be written entirely or not
- // at all.
- if (this->instantiated_target_ == NULL)
- {
- this->initialize_lock_.initialize();
- Hold_optional_lock hl(this->lock_);
- if (this->instantiated_target_ == NULL)
- this->instantiated_target_ = this->do_instantiate_target();
- }
+ this->set_target_once_.run_once(NULL);
return this->instantiated_target_;
}
+// Instantiate the target. This is called at most once.
+
+void
+Target_selector::set_target()
+{
+ gold_assert(this->instantiated_target_ == NULL);
+ this->instantiated_target_ = this->do_instantiate_target();
+}
+
// Find the target for an ELF file.
Target*
@@ -46,32 +54,31 @@ namespace gold
target_selectors = this;
}
-// Instantiate the target and return it. Use a lock to avoid
-// instantiating two instances of the same target.
+// Instantiate the target and return it. Use SET_TARGET_ONCE_ to
+// avoid instantiating two instances of the same target.
Target*
Target_selector::instantiate_target()